/*
 * IpatchDLS2Region.h - Header for DLS2 region object
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __IPATCH_DLS2_REGION_H__
#define __IPATCH_DLS2_REGION_H__

#include <stdarg.h>
#include <glib.h>
#include <glib-object.h>
#include <libinstpatch/IpatchItem.h>
#include <libinstpatch/IpatchDLS2Conn.h>
#include <libinstpatch/IpatchDLS2Sample.h>

/* forward type declarations */

typedef struct _IpatchDLS2Region IpatchDLS2Region;
typedef struct _IpatchDLS2RegionClass IpatchDLS2RegionClass;

#define IPATCH_TYPE_DLS2_REGION   (ipatch_dls2_region_get_type ())
#define IPATCH_DLS2_REGION(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_DLS2_REGION, \
  IpatchDLS2Region))
#define IPATCH_DLS2_REGION_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_DLS2_REGION, \
  IpatchDLS2RegionClass))
#define IPATCH_IS_DLS2_REGION(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_DLS2_REGION))
#define IPATCH_IS_DLS2_REGION_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_DLS2_REGION))
#define IPATCH_DLS2_REGION_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), IPATCH_TYPE_DLS2_REGION, \
  IpatchDLS2RegionClass))

/* DLS2 region item */
struct _IpatchDLS2Region
{
  IpatchItem parent_instance;

  /*< private >*/

  guint8 key_range_low;		/* MIDI key range low value */
  guint8 key_range_high;	/* MIDI key range high value */
  guint8 velocity_range_low;	/* MIDI velocity range low value */
  guint8 velocity_range_high;	/* MIDI velocity range high value */

  guint16 key_group;		/* Exclusive key group number or 0 */
  guint16 layer_group;		/* layer group (descriptive only) */

  guint16 phase_group;		/* Phase locked group number or 0 */
  guint16 channel;	/* channel ID (IpatchDLS2RegionChannelType) */

  IpatchDLS2Info *info;		/* info string values */
  IpatchDLS2SampleInfo *sample_info; /* sample info override or NULL */
  IpatchDLS2Sample *sample;	/* referenced sample */
  GSList *conns; /* list of connections (not used for GigaSampler objects) */
};

struct _IpatchDLS2RegionClass
{
  IpatchItemClass parent_class;
};

/* channel steering enum */
typedef enum /*< enum >*/
{
  IPATCH_DLS2_REGION_CHANNEL_LEFT = 0,
  IPATCH_DLS2_REGION_CHANNEL_RIGHT = 1,
  IPATCH_DLS2_REGION_CHANNEL_CENTER = 2,
  IPATCH_DLS2_REGION_CHANNEL_LOW_FREQ = 3,
  IPATCH_DLS2_REGION_CHANNEL_SURROUND_LEFT = 4,
  IPATCH_DLS2_REGION_CHANNEL_SURROUND_RIGHT = 5,
  IPATCH_DLS2_REGION_CHANNEL_LEFT_OF_CENTER = 6,
  IPATCH_DLS2_REGION_CHANNEL_RIGHT_OF_CENTER = 7,
  IPATCH_DLS2_REGION_CHANNEL_SURROUND_CENTER = 8,
  IPATCH_DLS2_REGION_CHANNEL_SIDE_LEFT = 9,
  IPATCH_DLS2_REGION_CHANNEL_SIDE_RIGHT = 10,
  IPATCH_DLS2_REGION_CHANNEL_TOP = 11,
  IPATCH_DLS2_REGION_CHANNEL_TOP_FRONT_LEFT = 12,
  IPATCH_DLS2_REGION_CHANNEL_TOP_FRONT_CENTER = 13,
  IPATCH_DLS2_REGION_CHANNEL_TOP_FRONT_RIGHT = 14,
  IPATCH_DLS2_REGION_CHANNEL_TOP_REAR_LEFT = 15,
  IPATCH_DLS2_REGION_CHANNEL_TOP_REAR_CENTER = 16,
  IPATCH_DLS2_REGION_CHANNEL_TOP_REAR_RIGHT = 17
} IpatchDLS2RegionChannelType;

/* mono audio alias */
#define IPATCH_DLS2_REGION_CHANNEL_MONO  IPATCH_DLS2_REGION_CHANNEL_LEFT

/* !! Keep order synchronized with sample info properties in
   IpatchDLS2Sample.c */
typedef enum /*< flags >*/
{
  IPATCH_DLS2_REGION_SAMPLE_OVERRIDE_FLAGS      = 1 << 0,
  IPATCH_DLS2_REGION_SAMPLE_OVERRIDE_LOOP       = 1 << 1,
  IPATCH_DLS2_REGION_SAMPLE_OVERRIDE_ROOT_NOTE  = 1 << 2,
  IPATCH_DLS2_REGION_SAMPLE_OVERRIDE_FINE_TUNE  = 1 << 3,
  IPATCH_DLS2_REGION_SAMPLE_OVERRIDE_GAIN       = 1 << 4,
  IPATCH_DLS2_REGION_SAMPLE_OVERRIDE_LOOP_START = 1 << 5,
  IPATCH_DLS2_REGION_SAMPLE_OVERRIDE_LOOP_SIZE  = 1 << 6
} IpatchDLS2RegionSampleOverride;

#define IPATCH_DLS2_REGION_SAMPLE_OVERRIDE_MASK  0x7F

/* Flags crammed into IpatchItem flags (ditched 2 - 16 bit flag fields) */
typedef enum /*< flags >*/
{
  IPATCH_DLS2_REGION_SELF_NON_EXCLUSIVE = 1 << IPATCH_ITEM_UNUSED_FLAG_SHIFT,
  IPATCH_DLS2_REGION_PHASE_MASTER = 1 << (IPATCH_ITEM_UNUSED_FLAG_SHIFT + 1),
  IPATCH_DLS2_REGION_MULTI_CHANNEL = 1 << (IPATCH_ITEM_UNUSED_FLAG_SHIFT + 2)
} IpatchDLS2RegionFlags;

#define IPATCH_DLS2_REGION_FLAG_MASK  (0x0F << IPATCH_ITEM_UNUSED_FLAG_SHIFT)

/* 3 flags + 1 for expansion + sample info override mask bits */
#define IPATCH_DLS2_REGION_UNUSED_FLAG_SHIFT \
  (IPATCH_ITEM_UNUSED_FLAG_SHIFT + 4 + IPATCH_DLS2_SAMPLE_INFO_PROPERTY_COUNT)

/* shift value for sample info override mask bits (IpatchItem flags) */
#define IPATCH_DLS2_REGION_SAMPLE_OVERRIDE_SHIFT \
  (IPATCH_ITEM_UNUSED_FLAG_SHIFT + 4)


GType ipatch_dls2_region_get_type (void);
IpatchDLS2Region *ipatch_dls2_region_new (void);

IpatchDLS2Region *ipatch_dls2_region_first (IpatchIter *iter);
IpatchDLS2Region *ipatch_dls2_region_next (IpatchIter *iter);

char *ipatch_dls2_region_get_info (IpatchDLS2Region *region, guint32 fourcc);
void ipatch_dls2_region_set_info (IpatchDLS2Region *region, guint32 fourcc,
				  const char *val);

void ipatch_dls2_region_set_sample (IpatchDLS2Region *region,
				    IpatchDLS2Sample *sample);
IpatchDLS2Sample *ipatch_dls2_region_ref_sample (IpatchDLS2Region *region);
IpatchDLS2Sample *ipatch_dls2_region_peek_sample (IpatchDLS2Region *region);

guint ipatch_dls2_region_get_sample_override (IpatchDLS2Region *region);
void ipatch_dls2_region_set_sample_override (IpatchDLS2Region *region,
					     guint mask);
void ipatch_dls2_region_set_sample_override_flags (IpatchDLS2Region *region,
						   guint flags);
void ipatch_dls2_region_clear_sample_override_flags (IpatchDLS2Region *region,
						     guint flags);

void ipatch_dls2_region_set_key_range (IpatchDLS2Region *region,
				       int low, int high);
void ipatch_dls2_region_set_velocity_range (IpatchDLS2Region *region,
					    int low, int high);
gboolean ipatch_dls2_region_in_range (IpatchDLS2Region *region, int note,
				      int velocity);

GSList *ipatch_dls2_region_get_conns (IpatchDLS2Region *region);
void ipatch_dls2_region_set_conn (IpatchDLS2Region *region,
				  const IpatchDLS2Conn *conn);
void ipatch_dls2_region_unset_conn (IpatchDLS2Region *region,
				    const IpatchDLS2Conn *conn);
void ipatch_dls2_region_unset_all_conns (IpatchDLS2Region *region);
guint ipatch_dls2_region_conn_count (IpatchDLS2Region *region);

int ipatch_dls2_region_channel_map_stereo (IpatchDLS2RegionChannelType chan);

#endif
