/*
 * IpatchDLSWriter.h - DLS patch writer
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __IPATCH_DLS_WRITER_H__
#define __IPATCH_DLS_WRITER_H__

#include <glib.h>
#include <libinstpatch/IpatchDLSFile.h>
#include <libinstpatch/IpatchRiffParser.h>
#include <libinstpatch/IpatchDLS2.h>

typedef struct _IpatchDLSWriter IpatchDLSWriter; /* private structure */
typedef struct _IpatchDLSWriterClass IpatchDLSWriterClass;

#define IPATCH_TYPE_DLS_WRITER   (ipatch_dls_writer_get_type ())
#define IPATCH_DLS_WRITER(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_DLS_WRITER, \
   IpatchDLSWriter))
#define IPATCH_DLS_WRITER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_DLS_WRITER, \
   IpatchDLSWriterClass))
#define IPATCH_IS_DLS_WRITER(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_DLS_WRITER))
#define IPATCH_IS_DLS_WRITER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_DLS_WRITER))

/* DLS writer object */
struct _IpatchDLSWriter
{
  IpatchRiffParser parent_instance; /* derived from IpatchRiffParser */
  IpatchDLS2 *orig_dls;		/* original DLS object */
  IpatchDLS2 *dls;		/* duplicated DLS object to save */
  gboolean is_gig;	/* set to TRUE if saving a GigaSampler file */
  GHashTable *sample_hash; /* IpatchDLS2Sample -> sample_array index + 1 */
  guint32 *sample_ofstbl;      /* sample index -> file offset table */
  guint sample_count;		/* count of samples */
  guint32 ptbl_pos;  /* pool table position in file - for later fixup */
};

/* DLS writer class */
struct _IpatchDLSWriterClass
{
  IpatchRiffParserClass parent_class;
};

GType ipatch_dls_writer_get_type (void);
IpatchDLSWriter *ipatch_dls_writer_new (IpatchDLSFile *file, IpatchDLS2 *dls);
void ipatch_dls_writer_set_patch (IpatchDLSWriter *writer, IpatchDLS2 *dls);
void ipatch_dls_writer_set_file (IpatchDLSWriter *writer, IpatchDLSFile *file);
gboolean ipatch_dls_writer_save (IpatchDLSWriter *writer, GError **err);

#endif
