/*
 * IpatchIfaceProp.h - Bind GObject style properties to interfaces
 *
 * GObject 2.6 supports interface object properties but not interface
 * class properties.  This implements interface class properties.
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_IFACE_PROP_H__
#define __IPATCH_IFACE_PROP_H__

#include <glib.h>
#include <glib-object.h>

/* GObject library doesn't really have a nice equivalent */
#define IPATCH_IS_INTERFACE(iface) G_TYPE_CHECK_INSTANCE_TYPE (iface, G_TYPE_INTERFACE)

/* useful macro for interface get/set property handlers */
#define IPATCH_IFACE_WARN_INVALID_PROPERTY_ID(iface, property_id, pspec) \
G_STMT_START { \
  GTypeInterface *_iface = (GTypeInterface *)(iface); \
  GParamSpec *_pspec = (GParamSpec*)(pspec); \
  guint _property_id = (property_id); \
  g_warning ("%s: invalid property id %u for \"%s\" of type `%s' in interface `%s'", \
             G_STRLOC, \
             _property_id, \
             _pspec->name, \
             g_type_name (G_PARAM_SPEC_TYPE (_pspec)), \
             g_type_name (G_TYPE_FROM_INTERFACE (_iface))); \
} G_STMT_END

/* definitions for interface class property handlers */
typedef void (*IpatchIfaceGetPropertyFunc)(GTypeInterface *iface,
	guint property_id, GValue *value, GParamSpec *pspec);
typedef void (*IpatchIfaceSetPropertyFunc)(GTypeInterface *iface,
	guint property_id, const GValue *value, GParamSpec *pspec);

void ipatch_iface_install_property_handlers (GTypeInterface *iface, 
	     IpatchIfaceGetPropertyFunc get_property,
	     IpatchIfaceSetPropertyFunc set_property);
void ipatch_iface_install_property (GTypeInterface *iface, guint property_id,
				    GParamSpec *prop_spec);
GParamSpec *ipatch_iface_find_property (GTypeInterface *iface,
					const char *name);
GParamSpec **ipatch_iface_list_properties (GTypeInterface *iface,
					   guint *n_properties);
void ipatch_iface_set (GTypeInterface *iface,
		       const char *first_property_name, ...);
void ipatch_iface_set_valist (GTypeInterface *iface,
			      const char *first_property_name, va_list args);
void ipatch_iface_set_property (GTypeInterface *iface,
				const char *property_name,
				const GValue *value);
void ipatch_iface_get (GTypeInterface *iface,
		       const char *first_property_name, ...);
void ipatch_iface_get_valist (GTypeInterface *iface,
			      const char *first_property_name,
			      va_list args);
void ipatch_iface_get_property (GTypeInterface *iface,
				const char *property_name,
				GValue *value);

#endif
