/*
 * IpatchParamProp.h - GParamSpec extended properties
 *
 * Extensions to standard GParamSpec include a flag registry (for
 * compact single bit data extensions) and GValue based extensions.
 * An example of usage is the IPATCH_PARAM_UNIQUE flag which indicates
 * a parameter that should be unique amongst sibling items and the
 * "string-max-length" integer GValue which specifies a max
 * length of a GParamSpecString parameter.
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_PARAM_PROP_H__
#define __IPATCH_PARAM_PROP_H__

#include <glib.h>
#include <glib-object.h>

/* GParamSpec flag to indicate parameter should be unique amongst siblings */
#define IPATCH_PARAM_UNIQUE  (1 << G_PARAM_USER_SHIFT)

/* !! NOTE: This value shouldn't be used directly - flag registry should be
   !! used instead (FIXME - Doesn't exist yet) */
#define IPATCH_PARAM_USER_SHIFT  (G_PARAM_USER_SHIFT + 8)


void ipatch_param_install_property (GParamSpec *prop_spec);
GParamSpec *ipatch_param_find_property (const char *name);
GParamSpec **ipatch_param_list_properties (guint *n_properties);
GParamSpec *ipatch_param_set (GParamSpec *spec,
			      const char *first_property_name, ...);
void ipatch_param_set_valist (GParamSpec *spec, const char *first_property_name,
			      va_list args);
void ipatch_param_set_property (GParamSpec *spec, const char *property_name,
				const GValue *value);
void ipatch_param_get (GParamSpec *spec, const char *first_property_name, ...);
void ipatch_param_get_valist (GParamSpec *spec, const char *first_property_name,
			      va_list args);
void ipatch_param_get_property (GParamSpec *spec, const char *property_name,
				GValue *value);

#endif
