/*
 * IpatchSF2Gen.h - Header for SoundFont generator routines (not an object)
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_SF2_GEN_H__
#define __IPATCH_SF2_GEN_H__

#include <glib.h>
#include <glib-object.h>

/* total number of generators */
#define IPATCH_SF2_GEN_COUNT 59

/* forward type declarations */
typedef union _IpatchSF2GenAmount IpatchSF2GenAmount;
typedef struct _IpatchSF2GenArray IpatchSF2GenArray;
typedef struct _IpatchSF2GenInfo IpatchSF2GenInfo;

/* IpatchSF2GenArray has a glib boxed type */
#define IPATCH_TYPE_SF2_GEN_ARRAY   (ipatch_sf2_gen_array_get_type ())

/* Generator amount (effect parameter amount) */
union _IpatchSF2GenAmount
{
  /*< public >*/
  gint16 sword;			/* signed 16 bit value */
  guint16 uword;		/* unsigned 16 bit value */
  struct
  {
    guint8 low;			/* low value of range */
    guint8 high;		/* high value of range */
  } range;			/* range values, low - high */
};

/* Generator (effect parameter) */
struct _IpatchSF2Gen
{
  /*< public >*/
  guint16 id;			/* generator #IPGenType ID */
  IpatchSF2GenAmount amount;	/* generator value */
};

/* generator array */
struct _IpatchSF2GenArray
{
  /*< public >*/
  guint64 flags; /* 1 bit for each generator indicating if it is set */
  IpatchSF2GenAmount values[IPATCH_SF2_GEN_COUNT]; /* gen values */
};

/* calculate the set bit value for a given generator ID */
#define IPATCH_SF2_GENID_SET(genid) ((guint64)0x1 << (genid))

/* macros for manipulating individual set flag bits in a generator array
   Note: These macros don't take into account multi-thread locking. */
#define IPATCH_SF2_GEN_ARRAY_TEST_FLAG(array, genid) \
  (((array)->flags & ((guint64)0x1 << (genid))) != 0)
#define IPATCH_SF2_GEN_ARRAY_SET_FLAG(array, genid) \
  ((array)->flags |= ((guint64)0x1 << (genid)))
#define IPATCH_SF2_GEN_ARRAY_CLEAR_FLAG(array, genid) \
  ((array)->flags &= ~((guint64)0x1 << (genid)))

/* generator (effect parameter) types */
typedef enum
{
  IPATCH_SF2_GEN_SAMPLE_START = 0, /* sample start offset */
  IPATCH_SF2_GEN_SAMPLE_END = 1,	/* sample end offset */
  IPATCH_SF2_GEN_SAMPLE_LOOP_START = 2,/* sample loop start offset */
  IPATCH_SF2_GEN_SAMPLE_LOOP_END = 3, /* sample loop end offset */
  IPATCH_SF2_GEN_SAMPLE_COARSE_START = 4, /* sample start coarse offset */
  IPATCH_SF2_GEN_MOD_LFO_TO_PITCH = 5, /* modulation LFO to pitch */
  IPATCH_SF2_GEN_VIB_LFO_TO_PITCH = 6, /* vibrato LFO to pitch */
  IPATCH_SF2_GEN_MOD_ENV_TO_PITCH = 7, /* modulation envelope to pitch */
  IPATCH_SF2_GEN_FILTER_FC = 8,	/* initial filter cutoff */
  IPATCH_SF2_GEN_FILTER_Q = 9,	/* filter Q */
  IPATCH_SF2_GEN_MOD_LFO_TO_FILTER_FC = 10, /* mod LFO to filter cutoff */
  IPATCH_SF2_GEN_MOD_ENV_TO_FILTER_FC = 11, /* mod envelope to filter cutoff */
  IPATCH_SF2_GEN_SAMPLE_COARSE_END = 12, /* sample end course offset */
  IPATCH_SF2_GEN_MOD_LFO_TO_VOL = 13, /* modulation LFO to volume */
  IPATCH_SF2_GEN_UNUSED1 = 14,
  IPATCH_SF2_GEN_CHORUS_SEND = 15, /* chorus */
  IPATCH_SF2_GEN_REVERB_SEND = 16, /* reverb */
  IPATCH_SF2_GEN_PAN = 17,	/* panning */
  IPATCH_SF2_GEN_UNUSED2 = 18,
  IPATCH_SF2_GEN_UNUSED3 = 19,
  IPATCH_SF2_GEN_UNUSED4 = 20,
  IPATCH_SF2_GEN_MOD_LFO_DELAY = 21, /* modulation LFO delay */
  IPATCH_SF2_GEN_MOD_LFO_FREQ = 22, /* modulation LFO frequency */
  IPATCH_SF2_GEN_VIB_LFO_DELAY = 23, /* vibrato LFO delay */
  IPATCH_SF2_GEN_VIB_LFO_FREQ = 24, /* vibrato LFO frequency */
  IPATCH_SF2_GEN_MOD_ENV_DELAY = 25, /* modulation envelope delay */
  IPATCH_SF2_GEN_MOD_ENV_ATTACK = 26, /* modulation envelope attack */
  IPATCH_SF2_GEN_MOD_ENV_HOLD = 27, /* modulation envelope hold */
  IPATCH_SF2_GEN_MOD_ENV_DECAY = 28, /* modulation envelope decay */
  IPATCH_SF2_GEN_MOD_ENV_SUSTAIN = 29, /* modulation envelope sustain */
  IPATCH_SF2_GEN_MOD_ENV_RELEASE = 30, /* modulation envelope release */
  IPATCH_SF2_GEN_KEY_TO_MOD_ENV_HOLD = 31, /* MIDI key to mod envelope hold */
  IPATCH_SF2_GEN_KEY_TO_MOD_ENV_DECAY = 32, /* MIDI key to mod env decay */
  IPATCH_SF2_GEN_VOL_ENV_DELAY = 33, /* volume envelope delay */
  IPATCH_SF2_GEN_VOL_ENV_ATTACK = 34, /* volume envelope attack */
  IPATCH_SF2_GEN_VOL_ENV_HOLD = 35, /* volume envelope hold */
  IPATCH_SF2_GEN_VOL_ENV_DECAY = 36, /* volume envelope decay */
  IPATCH_SF2_GEN_VOL_ENV_SUSTAIN = 37, /* volume envelope sustain */
  IPATCH_SF2_GEN_VOL_ENV_RELEASE = 38, /* volume envelope release */
  IPATCH_SF2_GEN_KEY_TO_VOL_ENV_HOLD = 39, /* MIDI key to vol envelope hold */
  IPATCH_SF2_GEN_KEY_TO_VOL_ENV_DECAY = 40, /* MIDI key to volume env decay */
  IPATCH_SF2_GEN_INSTRUMENT_ID = 41, /* instrument ID */
  IPATCH_SF2_GEN_RESERVED1 = 42,
  IPATCH_SF2_GEN_KEY_RANGE = 43,	/* note on key range */
  IPATCH_SF2_GEN_VELOCITY_RANGE = 44, /* note on velocity range */
  IPATCH_SF2_GEN_SAMPLE_COARSE_LOOP_START = 45, /* sample coarse loop start */
  IPATCH_SF2_GEN_FIXED_NOTE = 46, /* MIDI fixed note */
  IPATCH_SF2_GEN_FIXED_VELOCITY = 47, /* MIDI fixed velocity */
  IPATCH_SF2_GEN_ATTENUATION = 48, /* initial volume attenuation */
  IPATCH_SF2_GEN_RESERVED2 = 49,
  IPATCH_SF2_GEN_SAMPLE_COARSE_LOOP_END = 50, /* sample end loop course ofs */
  IPATCH_SF2_GEN_COARSE_TUNE = 51, /* course tuning */
  IPATCH_SF2_GEN_FINE_TUNE_OVERRIDE = 52, /* fine tune override */
  IPATCH_SF2_GEN_SAMPLE_ID = 53,	/* sample ID */
  IPATCH_SF2_GEN_SAMPLE_MODES = 54, /* sample flags (IpatchSF2GenSampleModes)*/
  IPATCH_SF2_GEN_RESERVED3 = 55,
  IPATCH_SF2_GEN_SCALE_TUNE = 56, /* scale tuning (tuning per MIDI key) */
  IPATCH_SF2_GEN_EXCLUSIVE_CLASS = 57, /* exclusive class (only 1 at a time) */
  IPATCH_SF2_GEN_ROOT_NOTE_OVERRIDE = 58 /* root note override */
} IpatchSF2GenType;

/* Flags for IPATCH_SF2_GEN_SAMPLE_MODES generator */
typedef enum
{
  IPATCH_SF2_GEN_SAMPLE_MODE_NOLOOP = 0,
  IPATCH_SF2_GEN_SAMPLE_MODE_LOOP   = 1 << 0,
  IPATCH_SF2_GEN_SAMPLE_MODE_LOOP_RELEASE = 1 << 1
} IpatchSF2GenSampleModes;

/* generator info and constraints structure */
struct _IpatchSF2GenInfo
{
  /*< public >*/
  IpatchSF2GenAmount min;	/* minimum value allowed */
  IpatchSF2GenAmount max;	/* maximum value allowed */
  IpatchSF2GenAmount def;	/* default value */
  gint16 unit;			/* #IpatchUnitType type */
  char *label;			/* short descriptive label */
  char *descr;			/* more complete description */
};

extern IpatchSF2GenArray *ipatch_sf2_gen_ofs_array;
extern IpatchSF2GenArray *ipatch_sf2_gen_abs_array;
extern guint64 ipatch_sf2_gen_ofs_valid_mask;
extern guint64 ipatch_sf2_gen_abs_valid_mask;
extern guint64 ipatch_sf2_gen_add_mask;
extern IpatchSF2GenInfo ipatch_sf2_gen_info[]; /* IpatchSF2Gen_tables.c */


GType ipatch_sf2_gen_array_get_type (void);
IpatchSF2GenArray *ipatch_sf2_gen_array_new (gboolean clear);
void ipatch_sf2_gen_array_free (IpatchSF2GenArray *genarray);
IpatchSF2GenArray *
ipatch_sf2_gen_array_duplicate (const IpatchSF2GenArray *array);

void ipatch_sf2_gen_array_init (IpatchSF2GenArray *array, gboolean offset,
				gboolean set);
void ipatch_sf2_gen_array_offset (IpatchSF2GenArray *abs_array,
				  const IpatchSF2GenArray *ofs_array);

void ipatch_sf2_gen_amount_to_value (guint genid, IpatchSF2GenAmount *amt,
				     GValue *value);

#define ipatch_sf2_genid_is_valid(genid, ispreset) \
 (((ispreset ? ipatch_sf2_gen_ofs_valid_mask : ipatch_sf2_gen_abs_valid_mask) \
  & ((guint64)0x1 << genid)) != 0)

void ipatch_sf2_genid_default_value (guint16 genid, gboolean ispreset,
				     IpatchSF2GenAmount *out_amt);
gboolean ipatch_sf2_genid_offset (int genid, IpatchSF2GenAmount *dst,
				  const IpatchSF2GenAmount *ofs);

void ipatch_sf2_units_sfont_to_user (guint16 genid,
				     const IpatchSF2GenAmount *amt,
				     float *out_userval, gboolean ispreset);
void ipatch_sf2_units_sfont_to_user_str (guint16 genid,
					 const IpatchSF2GenAmount *amt,
					 float *out_userval,
					 char **out_userstr,
					 gboolean ispreset);
int ipatch_sf2_units_user_to_sfont (guint16 genid, float userval,
				    gboolean ispreset, gboolean clamp);
void ipatch_sf2_units_clamp (guint16 genid, int *sfval, gboolean ispreset);
gboolean ipatch_sf2_units_range_intersect (IpatchSF2GenAmount *dst,
					   const IpatchSF2GenAmount *src);
char *ipatch_sf2_genid_unit_str (guint16 genid, gboolean ispreset);

#endif
