/*
 * IpatchSF2Reader.h - SoundFont 2 patch reader
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __IPATCH_SF2_READER_H__
#define __IPATCH_SF2_READER_H__

#include <glib.h>
#include <libinstpatch/IpatchSF2File.h>
#include <libinstpatch/IpatchSF2Gen.h>
#include <libinstpatch/IpatchSF2Mod.h>
#include <libinstpatch/IpatchRiffParser.h>
#include <libinstpatch/IpatchSF2.h>

typedef struct _IpatchSF2Reader IpatchSF2Reader; /* private structure */
typedef struct _IpatchSF2ReaderClass IpatchSF2ReaderClass;

#define IPATCH_TYPE_SF2_READER   (ipatch_sf2_reader_get_type ())
#define IPATCH_SF2_READER(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_SF2_READER, \
   IpatchSF2Reader))
#define IPATCH_SF2_READER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_SF2_READER, \
   IpatchSF2ReaderClass))
#define IPATCH_IS_SF2_READER(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_SF2_READER))
#define IPATCH_IS_SF2_READER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_SF2_READER))

/* SoundFont parser object */
struct _IpatchSF2Reader
{
  IpatchRiffParser parent_instance; /* derived from IpatchRiffParser */
  IpatchSF2 *sf;	      /* SoundFont object to load file into */

  /*< private >*/
  guint16 *pbag_table;
  guint pbag_count;
  guint16 *ibag_table;
  guint ibag_count;
  IpatchSF2Inst **inst_table;
  guint inst_count;
  IpatchSF2Sample **sample_table;
  guint sample_count;
};

/* RIFF parser class */
struct _IpatchSF2ReaderClass
{
  IpatchRiffParserClass parent_class;
};

GType ipatch_sf2_reader_get_type (void);
IpatchSF2Reader *ipatch_sf2_reader_new (IpatchSF2File *file);
void ipatch_sf2_reader_set_file (IpatchSF2Reader *reader, IpatchSF2File *file);
IpatchSF2 *ipatch_sf2_reader_load (IpatchSF2Reader *reader, GError **err);
void ipatch_sf2_load_phdr (IpatchFileBuf *filebuf, IpatchSF2Phdr *phdr);
void ipatch_sf2_load_ihdr (IpatchFileBuf *filebuf, IpatchSF2Ihdr *ihdr);
void ipatch_sf2_load_shdr (IpatchFileBuf *filebuf, IpatchSF2Shdr *shdr);
void ipatch_sf2_load_bag (IpatchFileBuf *filebuf, IpatchSF2Bag *bag);
void ipatch_sf2_load_mod (IpatchFileBuf *filebuf, IpatchSF2Mod *mod);
void ipatch_sf2_load_gen (IpatchFileBuf *filebuf, int *genid,
			  IpatchSF2GenAmount *amount);
#endif
