/*
 * IpatchSF2Sample.h - Header for SoundFont sample item
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_SF2_SAMPLE_H__
#define __IPATCH_SF2_SAMPLE_H__

#include <stdarg.h>
#include <glib.h>
#include <glib-object.h>
#include <libinstpatch/IpatchItem.h>
#include <libinstpatch/IpatchSampleData.h>

/* forward type declarations */

typedef struct _IpatchSF2Sample IpatchSF2Sample;
typedef struct _IpatchSF2SampleClass IpatchSF2SampleClass;

#define IPATCH_TYPE_SF2_SAMPLE   (ipatch_sf2_sample_get_type ())
#define IPATCH_SF2_SAMPLE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_SF2_SAMPLE, \
  IpatchSF2Sample))
#define IPATCH_SF2_SAMPLE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_SF2_SAMPLE, \
  IpatchSF2SampleClass))
#define IPATCH_IS_SF2_SAMPLE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_SF2_SAMPLE))
#define IPATCH_IS_SF2_SAMPLE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_SF2_SAMPLE))
#define IPATCH_SF2_SAMPLE_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), IPATCH_TYPE_SF2_SAMPLE, \
  IpatchSF2SampleClass))

/* SoundFont sample item */
struct _IpatchSF2Sample
{
  IpatchItem parent_instance;

  IpatchSampleData *sample_data; /* sample data object */
  char *name;			/* name of sample */
  guint32 loop_start;		/* loop start offset (in samples) */
  guint32 loop_end;		/* loop end offset (in samples, first sample
				   AFTER loop actually) */
  guint32 rate;			/* sample rate */
  guint8 root_note;		/* root midi note number */
  gint8 fine_tune;		/* fine tuning in cents */
  guint16 type;			/* IPATCH_SF2_SAMPLE_TYPE_* flags
				   (bit 1 mono, 2 right, 4 left, 8 linked,
				   0x8000 ROM) */
  IpatchSF2Sample *linked;	/* linked sample pointer or NULL */
};

struct _IpatchSF2SampleClass
{
  IpatchItemClass parent_class;
};

/* sampletype flag defines */

typedef enum /*< flags >*/
{
  IPATCH_SF2_SAMPLE_MONO	= 1 << 0,
  IPATCH_SF2_SAMPLE_RIGHT	= 1 << 1,
  IPATCH_SF2_SAMPLE_LEFT	= 1 << 2,
  IPATCH_SF2_SAMPLE_LINKED	= 1 << 3,
  IPATCH_SF2_SAMPLE_ROM	= 0x8000
} IpatchSF2SampleFlags;

/* sample rate and length constraints */

#define IPATCH_SF2_SAMPLE_RATE_MIN	400 /* min sample rate (by standard) */
#define IPATCH_SF2_SAMPLE_RATE_MAX	50000 /* max rate (by the standard) */
#define IPATCH_SF2_SAMPLE_LENGTH_MIN	32 /* min length (by the standard) */

GType ipatch_sf2_sample_get_type (void);
IpatchSF2Sample *ipatch_sf2_sample_new (void);

IpatchSF2Sample *ipatch_sf2_sample_first (IpatchIter *iter);
IpatchSF2Sample *ipatch_sf2_sample_next (IpatchIter *iter);

void ipatch_sf2_sample_set_name (IpatchSF2Sample *sample, const char *name);
char *ipatch_sf2_sample_get_name (IpatchSF2Sample *sample);
void ipatch_sf2_sample_set_data (IpatchSF2Sample *sample,
				 IpatchSampleData *sampledata);
IpatchSampleData *ipatch_sf2_sample_ref_data (IpatchSF2Sample *sample);
IpatchSampleData *ipatch_sf2_sample_peek_data (IpatchSF2Sample *sample);
void ipatch_sf2_sample_set_linked (IpatchSF2Sample *sample,
				   IpatchSF2Sample *linked);
IpatchSF2Sample *ipatch_sf2_sample_ref_linked (IpatchSF2Sample *sample);
IpatchSF2Sample *ipatch_sf2_sample_peek_linked (IpatchSF2Sample *sample);
void ipatch_sf2_sample_set_blank (IpatchSF2Sample *sample);
guint ipatch_sf2_sample_get_size (IpatchSF2Sample *sample);
IpatchSampleStore *ipatch_sf2_sample_find_store_ref (IpatchSF2Sample *sample,
						     GType store_type,
						     int format,
						     int find_flags);
#endif
