/*
 * IpatchSF2Writer.h - SF2 patch writer
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __IPATCH_SF2_WRITER_H__
#define __IPATCH_SF2_WRITER_H__

#include <glib.h>
#include <libinstpatch/IpatchSF2File.h>
#include <libinstpatch/IpatchRiffParser.h>
#include <libinstpatch/IpatchSF2.h>
#include <libinstpatch/IpatchSF2Mod.h>
#include <libinstpatch/IpatchSF2Gen.h>

typedef struct _IpatchSF2Writer IpatchSF2Writer; /* private structure */
typedef struct _IpatchSF2WriterClass IpatchSF2WriterClass;

#define IPATCH_TYPE_SF2_WRITER   (ipatch_sf2_writer_get_type ())
#define IPATCH_SF2_WRITER(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_SF2_WRITER, \
   IpatchSF2Writer))
#define IPATCH_SF2_WRITER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_SF2_WRITER, \
   IpatchSF2WriterClass))
#define IPATCH_IS_SF2_WRITER(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_SF2_WRITER))
#define IPATCH_IS_SF2_WRITER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_SF2_WRITER))

/* SF2 writer object */
struct _IpatchSF2Writer
{
  IpatchRiffParser parent_instance; /* derived from IpatchRiffParser */
  IpatchSF2 *orig_sf;		/* original SF2 object */
  IpatchSF2 *sf;		/* duplicated SF2 object to save */
  gboolean migrate_samples; /* set to TRUE to migrate samples to new file */
  gboolean temp_stores;	/* set if any temp sample stores have been created */
  GHashTable *inst_hash;	/* instrument => index hash */
  GHashTable *sample_hash;	/* sample => index hash */
};

/* SF2 writer class */
struct _IpatchSF2WriterClass
{
  IpatchRiffParserClass parent_class;
};

GType ipatch_sf2_writer_get_type (void);
IpatchSF2Writer *ipatch_sf2_writer_new (IpatchSF2File *file, IpatchSF2 *sfont);
void ipatch_sf2_writer_set_patch (IpatchSF2Writer *writer, IpatchSF2 *sfont);
void ipatch_sf2_writer_set_file (IpatchSF2Writer *writer, IpatchSF2File *file);
gboolean ipatch_sf2_writer_save (IpatchSF2Writer *writer, GError **err);

void ipatch_sf2_write_phdr (IpatchFileBuf *filebuf, const IpatchSF2Phdr *phdr);
void ipatch_sf2_write_ihdr (IpatchFileBuf *filebuf, const IpatchSF2Ihdr *ihdr);
void ipatch_sf2_write_shdr (IpatchFileBuf *filebuf, const IpatchSF2Shdr *shdr);
void ipatch_sf2_write_bag (IpatchFileBuf *filebuf, const IpatchSF2Bag *bag);
void ipatch_sf2_write_mod (IpatchFileBuf *filebuf, const IpatchSF2Mod *mod);
void ipatch_sf2_write_gen (IpatchFileBuf *filebuf, int genid,
			   const IpatchSF2GenAmount *amount);

#endif
