/*
 * IpatchSF2Zone.h - Header for SoundFont zone base class
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_SF2_ZONE_H__
#define __IPATCH_SF2_ZONE_H__

#include <stdarg.h>
#include <glib.h>
#include <glib-object.h>
#include <libinstpatch/IpatchItem.h>
#include <libinstpatch/IpatchSF2Gen.h>
#include <libinstpatch/IpatchSF2Mod.h>

/* forward type declarations */

typedef struct _IpatchSF2Zone IpatchSF2Zone;
typedef struct _IpatchSF2ZoneClass IpatchSF2ZoneClass;

#define IPATCH_TYPE_SF2_ZONE   (ipatch_sf2_zone_get_type ())
#define IPATCH_SF2_ZONE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_SF2_ZONE, \
  IpatchSF2Zone))
#define IPATCH_SF2_ZONE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_SF2_ZONE, \
  IpatchSF2ZoneClass))
#define IPATCH_IS_SF2_ZONE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_SF2_ZONE))
#define IPATCH_IS_SF2_ZONE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_SF2_ZONE))
#define IPATCH_SF2_ZONE_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), IPATCH_TYPE_SF2_ZONE, \
  IpatchSF2ZoneClass))

/* SoundFont zone item */
struct _IpatchSF2Zone
{
  IpatchItem parent_instance;

  /*< private >*/

  IpatchItem *item;		/* referenced item or NULL if global zone */
  GSList *mods;			/* modulators */
  IpatchSF2GenArray genarray;	/* generator array */
};

struct _IpatchSF2ZoneClass
{
  IpatchItemClass parent_class;
};

/* a macro for getting or setting a generator value directly, normally
   ipatch_sf2_zone_(get/set)_gen should be used instead, zone is not locked
   and no flags are set (should only be used on zones with exclusive access) */
#define IPATCH_SF2_ZONE_GEN_AMT(zone, genid) \
  ((IpatchSF2Zone *)zone)->genarray.values[genid]

/* macros for manipulating zone generator set flags directly, should not
   normally be used except in the case of exclusive access, not locked, etc */
#define IPATCH_SF2_ZONE_GEN_TEST_FLAG(zone, genid) \
  IPATCH_SF2_GEN_ARRAY_TEST_FLAG (((IpatchSF2Zone *)zone)->array, genid)
#define IPATCH_SF2_ZONE_SET_FLAG(zone, genid) \
  IPATCH_SF2_GEN_ARRAY_SET_FLAG (((IpatchSF2Zone *)zone)->array, genid)
#define IPATCH_SF2_ZONE_CLEAR_FLAG(zone, genid) \
  IPATCH_SF2_GEN_ARRAY_CLEAR_FLAG (((IpatchSF2Zone *)zone)->array, genid)


#define ipatch_sf2_zone_add_mod(zone, mod) \
  ipatch_sf2_zone_insert_mod (zone, mod, -1)

GType ipatch_sf2_zone_get_type (void);

IpatchSF2Zone *ipatch_sf2_zone_first (IpatchIter *iter);
IpatchSF2Zone *ipatch_sf2_zone_next (IpatchIter *iter);

void ipatch_sf2_zone_set_item (IpatchSF2Zone *zone, IpatchItem *item);
IpatchItem *ipatch_sf2_zone_ref_item (IpatchSF2Zone *zone);
IpatchItem *ipatch_sf2_zone_peek_item (IpatchSF2Zone *zone);

gboolean ipatch_sf2_zone_get_gen (IpatchSF2Zone *zone, int genid,
				  IpatchSF2GenAmount *out_amt);
void ipatch_sf2_zone_set_gen (IpatchSF2Zone *zone, int genid,
			      IpatchSF2GenAmount *amt);
void ipatch_sf2_zone_unset_gen (IpatchSF2Zone *zone, int genid);
guint ipatch_sf2_zone_gen_count (IpatchSF2Zone *zone);

G_CONST_RETURN char *ipatch_sf2_zone_genid_get_prop_name (guint genid);
GParamSpec *ipatch_sf2_zone_genid_get_pspec (guint genid, gboolean ispreset);

void ipatch_sf2_zone_copy_gen_array (IpatchSF2Zone *zone,
				     IpatchSF2GenArray *array);
void ipatch_sf2_zone_fill_gen_array (IpatchSF2Zone *zone,
				     IpatchSF2GenArray *array);

GSList *ipatch_sf2_zone_get_mods (IpatchSF2Zone *zone);
void ipatch_sf2_zone_insert_mod (IpatchSF2Zone *zone, const IpatchSF2Mod *mod,
				 int pos);
void ipatch_sf2_zone_remove_mod (IpatchSF2Zone *zone,
				 const IpatchSF2Mod *modvals);
void ipatch_sf2_zone_set_mod (IpatchSF2Zone *zone, const IpatchSF2Mod *oldvals,
			      const IpatchSF2Mod *newvals);
guint ipatch_sf2_zone_mod_count (IpatchSF2Zone *zone);

void ipatch_sf2_zone_set_key_range (IpatchSF2Zone *zone, int low, int high);
void ipatch_sf2_zone_set_velocity_range (IpatchSF2Zone *zone,
                                         int low, int high);
gboolean ipatch_sf2_zone_in_range (IpatchSF2Zone *zone, int note,
				   int velocity);
void ipatch_sf2_zone_class_install_gen_properties (GObjectClass *klass,
						   gboolean ispreset);
gboolean ipatch_sf2_zone_set_gen_property (IpatchSF2Zone *zone,
					   guint property_id,
					   const GValue *value,
					   gboolean is_preset);
gboolean ipatch_sf2_zone_get_gen_property (IpatchSF2Zone *zone,
					   guint property_id,
					   GValue *value, gboolean is_preset);
#endif
