/*
 * IpatchSampleData.h - Header for sample data object
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_SAMPLE_DATA_H__
#define __IPATCH_SAMPLE_DATA_H__

#include <stdarg.h>
#include <glib.h>
#include <glib-object.h>
#include <libinstpatch/IpatchContainer.h>
#include <libinstpatch/IpatchSampleStore.h>
#include <libinstpatch/misc.h>

/* forward type declarations */

typedef struct _IpatchSampleData IpatchSampleData;
typedef struct _IpatchSampleDataClass IpatchSampleDataClass;

#define IPATCH_TYPE_SAMPLE_DATA  (ipatch_sample_data_get_type ())
#define IPATCH_SAMPLE_DATA(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_SAMPLE_DATA, \
  IpatchSampleData))
#define IPATCH_SAMPLE_DATA_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_SAMPLE_DATA, \
  IpatchSampleDataClass))
#define IPATCH_IS_SAMPLE_DATA(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_SAMPLE_DATA))
#define IPATCH_IS_SAMPLE_DATA_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_SAMPLE_DATA))
#define IPATCH_SAMPLE_DATA_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), IPATCH_TYPE_SAMPLE_DATA, \
  IpatchSampleDataClass))

/* sample data instance */
struct _IpatchSampleData
{
  IpatchContainer parent_instance;

  guint32 size;			/* size of sample data (in samples) */
  guint32 rate;			/* sample rate in hz */
  GSList *stores;		/* sample stores (IpatchSampleStore) */
};

struct _IpatchSampleDataClass
{
  IpatchContainerClass parent_class;
};

/* reserve 3 flags for expansion */
#define IPATCH_SAMPLE_DATA_UNUSED_FLAG_SHIFT \
  (IPATCH_CONTAINER_UNUSED_FLAG_SHIFT + 3)


/* flags for finding IpatchSampleStore objects with
   ipatch_sample_data_find_store_ref () */
typedef enum /*< flags >*/
{
  IPATCH_SAMPLE_DATA_FIND_READABLE     = 1 << 0,
  IPATCH_SAMPLE_DATA_FIND_WRITABLE     = 1 << 1,
  IPATCH_SAMPLE_DATA_FIND_FASTEST      = 1 << 2,
  IPATCH_SAMPLE_DATA_FIND_INACTIVE     = 1 << 3,
  IPATCH_SAMPLE_DATA_FIND_EXACT_FORMAT = 1 << 4
} IpatchSampleDataFindFlags;

GType ipatch_sample_data_get_type (void);
IpatchSampleData *ipatch_sample_data_new (void);

IpatchSampleData *ipatch_sample_data_first (IpatchIter *iter);
IpatchSampleData *ipatch_sample_data_next (IpatchIter *iter);

#define ipatch_sample_data_get_stores(sampledata) \
    ipatch_container_get_children (IPATCH_CONTAINER (sampledata), \
				   IPATCH_TYPE_SAMPLE_STORE)

void ipatch_sample_data_set_size (IpatchSampleData *sampledata, guint size);
guint ipatch_sample_data_get_size (IpatchSampleData *sampledata);

IpatchSampleStore *
ipatch_sample_data_ref_primary_store (IpatchSampleData *sampledata);
int ipatch_sample_data_get_primary_format (IpatchSampleData *sampledata);
int ipatch_sample_data_get_primary_channels (IpatchSampleData *sampledata);
int ipatch_sample_data_get_primary_frame_size (IpatchSampleData *sampledata);

IpatchSampleStore *
ipatch_sample_data_find_store_ref (IpatchSampleData *sampledata,
				   GType store_type, int format,
				   int find_flags);
IpatchSampleData *ipatch_sample_data_ref_blank (void);
IpatchList *ipatch_sample_data_get_list (void);

#endif
