/*
 * IpatchSampleFile.h - Base sample file object class (sample lib interface)
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __IPATCH_SAMPLE_FILE_H__
#define __IPATCH_SAMPLE_FILE_H__

#include <glib.h>
#include <glib-object.h>
#include <libinstpatch/IpatchFile.h>
#include <libinstpatch/IpatchRiffParser.h>
#include <libinstpatch/sample.h>

/*
 * The IpatchSampleFile object defines a base type derived from
 * IpatchFile to read/write sample files. From this new
 * IpatchSampleFile derived objects will be created that interface to
 * external sample libraries like libsndfile or audiofile. Most of the
 * "magic" occurs at the IpatchFile layer where the io functions are
 * overridden.
 * Only one IpatchSampleFile derived library interface type gets selected
 * for use with sample files.
 */

#include <libinstpatch/IpatchSample.h>

/* forward type declarations */

typedef struct _IpatchSampleFile IpatchSampleFile;
typedef struct _IpatchSampleFileClass IpatchSampleFileClass;

#define IPATCH_TYPE_SAMPLE_FILE   (ipatch_sample_file_get_type ())
#define IPATCH_SAMPLE_FILE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_SAMPLE_FILE, \
   IpatchSampleFile))
#define IPATCH_SAMPLE_FILE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_SAMPLE_FILE, \
  IpatchSampleFileClass))
#define IPATCH_IS_SAMPLE_FILE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_SAMPLE_FILE))
#define IPATCH_IS_SAMPLE_FILE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), IPATCH_TYPE_SAMPLE_FILE))
#define IPATCH_SAMPLE_FILE_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), IPATCH_TYPE_SAMPLE_FILE, \
  IpatchSampleFileClass))

/* sample file object (derived from IpatchFile) */
struct _IpatchSampleFile
{
  IpatchFile parent_instance;

  char type[8];			/* file type string (read/write mode) */
  guint size;			/* size of file in frames (read mode only) */

  /* file sample format variables (read/write) */
  int format;	/* format of sample data (see sample.h) */
  int rate;		   /* sampling rate of audio (0 if unknown) */

  guint frame_size; /* size in bytes of a single audio frame */

  /* instrument data (read and write mode) */
  IpatchSampleLoopType loop_type; /* loop type */
  guint loop_start;		/* start of loop in instrument */
  guint loop_end;		/* end of loop in instrument */
  int root_note;		/* MIDI root note (-1 if not set) */
  int fine_tune;		/* fine tuning of sample in cents */
};

/* sample file class (derived from IpatchFile) */
struct _IpatchSampleFileClass
{
  IpatchFileClass parent_class;

  G_CONST_RETURN char ** (*get_save_types)(void);
};

/* file type strings (the more useful ones to write to) */
#define IPATCH_SAMPLE_FILE_TYPE_RAW   "raw" /* special type */
#define IPATCH_SAMPLE_FILE_TYPE_WAVE  "wave" /* default type */

GType ipatch_sample_file_get_type (void);
G_CONST_RETURN char **ipatch_sample_file_get_save_types (GType type);

#endif
