/*
 * IpatchSampleTransform.h - Header for sample transform object
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 */
#ifndef __IPATCH_SAMPLE_TRANSFORM_H__
#define __IPATCH_SAMPLE_TRANSFORM_H__

#include <stdio.h>
#include <glib.h>
#include <glib-object.h>

typedef struct _IpatchSampleTransform IpatchSampleTransform;
typedef struct _IpatchSampleTransformClass IpatchSampleTransformClass;

#define IPATCH_TYPE_SAMPLE_TRANSFORM   (ipatch_sample_transform_get_type ())
#define IPATCH_SAMPLE_TRANSFORM(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_SAMPLE_TRANSFORM, \
   IpatchSampleTransform))
#define IPATCH_IS_SAMPLE_TRANSFORM(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_SAMPLE_TRANSFORM))

/**
 * IpatchSampleTransformFunc:
 * @transform: Sample transform object
 *
 * Audio conversion handler prototype. Handler will modify
 * <structfield>samples</structfield> of @transform if number of samples
 * changes (a change in the number of channels occurs).
 */
typedef void (*IpatchSampleTransformFunc)(IpatchSampleTransform *transform);

/* sample transform object */
struct _IpatchSampleTransform
{
  GObject parent_instance;

  guint16 src_format;		/* source sample format */
  guint16 dest_format;		/* destination sample format */
  guint8 buf1_max_frame;	/* max bytes per frame for buf1 */
  guint8 buf2_max_frame;	/* max bytes per frame for buf2 */
  guint8 func_count;		/* number of functions in funcs array */
  guint8 free_buffers;		/* set to TRUE if buffers need to be freed */
  guint max_frames;		/* max frames that can be converted in batch */
  guint frames;			/* number of frames to transform */
  guint samples; /* # of samples for current transform func (not frames!)*/
  gpointer buf1;		/* buffer 1 (first input) */
  gpointer buf2;		/* buffer 2 */
  IpatchSampleTransformFunc *funcs; /* array of transform funcs */
};

struct _IpatchSampleTransformClass
{
  /*< private >*/
  GObjectClass parent_class;
};


GType ipatch_sample_transform_get_type (void);
IpatchSampleTransform *ipatch_sample_transform_new (int src_format,
						    int dest_format);
void ipatch_sample_transform_set_formats (IpatchSampleTransform *transform,
					  int src_format, int dest_format);
void ipatch_sample_transform_alloc (IpatchSampleTransform *transform,
				    guint frames);
int ipatch_sample_transform_alloc_size (IpatchSampleTransform *transform,
					guint size);
void ipatch_sample_transform_free_buffers (IpatchSampleTransform *transform);
void ipatch_sample_transform_set_buffers (IpatchSampleTransform *transform,
					  gpointer buf1, gpointer buf2,
					  guint max_frames);
guint ipatch_sample_transform_set_buffers_size (IpatchSampleTransform *transform,
						gpointer buf, guint size);
void ipatch_sample_transform_get_buffers (IpatchSampleTransform *transform,
					  gpointer *buf1, gpointer *buf2);
void ipatch_sample_transform_get_frame_sizes (IpatchSampleTransform *transform,
					      guint *buf1_size,
					      guint *buf2_size);
guint ipatch_sample_transform_get_max_frames (IpatchSampleTransform *transform);
gpointer ipatch_sample_transform_convert (IpatchSampleTransform *transform,
					  guint frames);
#endif
