/*
 * IpatchVirtualContainer.h - Header for virtual container object
 * Virtual containers are used in user interfaces to group items in
 * containers that aren't actually present in the hierarchy, such as
 * "Instruments", "Melodic Presets", "Percussion Presets" in SF2 files.
 *
 * libInstPatch
 * Copyright (C) 1999-2005 Josh Green <jgreen@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA or point your web browser to http://www.gnu.org.
 *
 */
#ifndef __IPATCH_VIRTUAL_CONTAINER_H__
#define __IPATCH_VIRTUAL_CONTAINER_H__

#include <glib.h>
#include <glib-object.h>

/* forward type declarations */

typedef struct _IpatchVirtualContainer IpatchVirtualContainer;
typedef struct _IpatchVirtualContainerClass IpatchVirtualContainerClass;

#include <libinstpatch/IpatchItem.h>

#define IPATCH_TYPE_VIRTUAL_CONTAINER   (ipatch_virtual_container_get_type ())
#define IPATCH_VIRTUAL_CONTAINER(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), IPATCH_TYPE_VIRTUAL_CONTAINER, \
   IpatchVirtualContainer))
#define IPATCH_VIRTUAL_CONTAINER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), IPATCH_TYPE_VIRTUAL_CONTAINER, \
   IpatchVirtualContainerClass))
#define IPATCH_IS_VIRTUAL_CONTAINER(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), IPATCH_TYPE_VIRTUAL_CONTAINER))

/* virtual container object */
struct _IpatchVirtualContainer
{
  IpatchItem parent_instance; /* derived from IpatchItem */
};

/* virtual container class */
struct _IpatchVirtualContainerClass
{
  IpatchItemClass parent_class;
};

GType ipatch_virtual_container_get_type (void);

#endif
